# Copyright 2021 - 2023 Universität Tübingen, DKFZ, EMBL, and Universität zu Köln
# for the German Human Genome-Phenome Archive (GHGA)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Config parameters and their defaults."""


from metldata.event_handling.event_handling import FileSystemEventConfig
from metldata.transform.artifact_publisher import ArtifactEventPublisherConfig
from metldata.transform.source_event_subscriber import SourceEventSubscriberConfig


# pylint: disable=too-many-ancestors
class TransformationEventHandlingConfig(
    FileSystemEventConfig, ArtifactEventPublisherConfig, SourceEventSubscriberConfig
):
    """Config parameters for consuming source events and publishing artifacts."""



# Copyright 2021 - 2023 Universität Tübingen, DKFZ, EMBL, and Universität zu Köln
# for the German Human Genome-Phenome Archive (GHGA)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""The CLI interface."""

import asyncio
import functools
from pathlib import Path

import typer
import yaml
from ghga_service_commons.api import run_server

from metldata.config import ArtifactLoaderAPIConfig, ArtifactLoaderClientConfig
from metldata.load.client import upload_artifacts_via_http_api
from metldata.load.main import get_app

cli = typer.Typer()


def run_sync(coroutine):
    """A decorator to run a coroutine as a synchronous function."""

    @functools.wraps(coroutine)
    def wrapper(*args, **kwargs):
        return asyncio.run(coroutine(*args, **kwargs))

    return wrapper


@cli.command()
@run_sync
async def run_loader_api() -> None:
    """Run the loader API."""

    config = ArtifactLoaderAPIConfig()
    app = await get_app(config=config)
    await run_server(app=app, config=config)


@cli.command()
def upload_artifact(*, config: Path) -> None:
    """Upload an artifact to the loader API.

    Args:
        config: The path to the config file.
    """

    with open(config, "r", encoding="utf-8") as config_file:
        config_dict = yaml.safe_load(config_file)
    config_obj = ArtifactLoaderClientConfig(**config_dict)

    token = input(
        "Please paste the token used to authenticate against the loader API: "
    )

    upload_artifacts_via_http_api(config=config_obj, token=token.strip())

    typer.echo("Done.")
